/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.dialogs;

import java.io.File;
import java.io.FileFilter;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.TreeEvent;
import org.eclipse.swt.events.TreeListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;

public class SelectDirDialog extends Dialog {
    private final static int TREE_WIDTH = 250;
    private final static int TREE_HEIGHT = 300;
    private final static String TITLE = "Folder selection";

    private File root = null;
    private String path = "";
    private boolean select = true;

    private TreeItem selectedItem = null;
    private File selectedDir = null;
    private Tree tree = null;
    private Text makeText = null;

    private Button makeBut = null;

    protected static FileFilter directoryFilter = new FileFilter() {
        public boolean accept(File f) {
            return f.isDirectory();
        }
    };

    private TreeListener treeLink = new TreeListener() {
        public void createSubtree(TreeItem rootItem, File rootDir,
                boolean select) {
            if (rootItem != null) {
                rootItem.removeAll();
                if (rootDir != null) {
                    File[] children = rootDir.listFiles(directoryFilter);
                    for (File child : children) {
                        TreeItem item = new TreeItem(rootItem, SWT.NONE);
                        item.setImage(PlatformUI.getWorkbench()
                                .getSharedImages()
                                .getImage(ISharedImages.IMG_OBJ_FOLDER));
                        item.setText(child.getName());
                        item.setData(child);
                        if (child.listFiles(directoryFilter).length > 0)
                            item.setItemCount(1);
                        String dirPath = child.getPath();
                        if (select == true) {
                            if (path.startsWith(dirPath)) {
                                if (path.equals(dirPath) == true) {
                                    selectedItem = item;
                                    selectedDir = child;
                                } else {
                                    createSubtree(item, child, select);
                                    item.setExpanded(true);
                                }
                            }
                        }
                    }
                }
            }
        }

        public void treeCollapsed(TreeEvent e) {
        }

        public void treeExpanded(TreeEvent e) {
            if (e.item instanceof TreeItem) {
                TreeItem rootItem = (TreeItem) e.item;
                if (rootItem.getData() instanceof File) {
                    File rootDir = (File) rootItem.getData();
                    createSubtree(rootItem, rootDir, select);
                    select = false;
                }
            }
        }
    };

    private SelectionListener selectList = new SelectionListener() {
        public void widgetDefaultSelected(SelectionEvent e) {
        }

        public void widgetSelected(SelectionEvent e) {
            if (e.item instanceof TreeItem) {
                TreeItem item = (TreeItem) e.item;
                if (item.getData() instanceof File) {
                    selectedDir = (File) item.getData();
                }
            }
        }
    };

    private MouseListener mouseList = new MouseListener() {
        public void mouseDoubleClick(MouseEvent e) {
            TreeItem item = tree.getItem(new Point(e.x, e.y)); // @jve:decl-index=0:
            if (item != null)
                okPressed();
        }

        public void mouseDown(MouseEvent e) {
        }

        public void mouseUp(MouseEvent e) {
        }
    };

    private KeyListener keyList = new KeyListener() {
        public void keyPressed(KeyEvent e) {
            if (e.keyCode == ' ') {
                TreeItem[] items = tree.getSelection();
                if (items.length > 0) {
                    TreeItem item = items[0];
                    if (item.getExpanded() == false) {
                        item.setExpanded(true);
                        Event ev = new Event();
                        ev.item = item;
                        tree.notifyListeners(SWT.Expand, ev);
                    } else
                        item.setExpanded(false);
                }
            }
            if (e.keyCode == SWT.CR) {
                okPressed();
            }
            if (e.keyCode == SWT.INSERT) {
                makeText.setFocus();
                makeText.setText("");
            }
            if (e.keyCode == SWT.DEL) {
                if (tree.getSelectionCount() > 0) {
                    TreeItem[] sels = tree.getSelection();
                    String mes = (sels.length > 1) ? "Are you sure you want to remove these folders?"
                            : "Are you sure you want to delete this folder?";
                    if (MessageDialog.openQuestion(getShell(),
                            "Folder selection", mes) == true) {
                        for (TreeItem sel : sels) {
                            if (sel.getData() instanceof File) {
                                File dir = (File) sel.getData();
                                if (dir.exists() && !dir.equals(root)) {
                                    if (dir.listFiles().length == 0) {
                                        dir.delete();
                                        TreeItem parent = sel.getParentItem();
                                        if (parent != null) {
                                            parent.setExpanded(true);
                                            Event ev = new Event();
                                            ev.item = parent;
                                            tree.notifyListeners(SWT.Expand, ev);
                                        }
                                    } else {
                                        mes = "The folder must be empty.";
                                        MessageDialog.openInformation(
                                                getShell(), "Folder selection",
                                                mes);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        public void keyReleased(KeyEvent e) {
        }
    };

    public SelectDirDialog(Shell shell, File root, String path) {
        super(shell);
        this.root = root;
        // this.site = site;
        if (path != null)
            this.path = path;
    }

    public File getResult() {
        return selectedDir;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = createClientContainer(parent, 1, -1);

        Label commentLabel = new Label(container, SWT.NONE);
        commentLabel.setText("Select a folder:");
        commentLabel.setLayoutData(new GridData(GridData.BEGINNING,
                GridData.CENTER, true, true));

        tree = new Tree(container, SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
        tree.setLayoutData(new GridData(TREE_WIDTH, TREE_HEIGHT));
        tree.addTreeListener(treeLink);
        tree.addSelectionListener(selectList);
        tree.addMouseListener(mouseList);
        tree.addKeyListener(keyList);

        if ((root != null) && (root.isDirectory())) {
            TreeItem rootItem = new TreeItem(tree, SWT.NONE);
            rootItem.setText(root.getName());
            rootItem.setImage(PlatformUI.getWorkbench().getSharedImages()
                    .getImage(ISharedImages.IMG_OBJ_FOLDER));
            rootItem.setData(root);
            if (root.exists() && (root.listFiles(directoryFilter).length > 0))
                rootItem.setItemCount(1);
            if (path.length() > 0) {
                rootItem.setExpanded(true);
                Event e = new Event();
                e.item = rootItem;
                e.widget = tree;
                treeLink.treeExpanded(new TreeEvent(e));
                if (selectedItem != null)
                    tree.setSelection(selectedItem);
            }
        }

        Composite mkDir = createClientContainer(container, 3, 0);

        Label makeLabel = new Label(mkDir, SWT.NONE);
        makeLabel.setText("Folder:");

        KeyListener makeKey = new KeyListener() {
            public void keyPressed(KeyEvent e) {
                if ((e.keyCode == SWT.CR) && makeText.isFocusControl()) {
                    makeBut.notifyListeners(SWT.Selection, new Event());
                }
            }

            public void keyReleased(KeyEvent e) {
            }
        };

        makeText = new Text(mkDir, SWT.BORDER);
        makeText.setLayoutData(new GridData(180, -1));
        makeText.addKeyListener(makeKey);

        SelectionListener makeClick = new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
            }

            public void widgetSelected(SelectionEvent e) {
                String name = makeText.getText();
                if ((tree.getSelectionCount() > 0) && (name.length() > 0)) {
                    TreeItem sel = tree.getSelection()[0];
                    if (sel.getData() instanceof File) {
                        File parent = (File) sel.getData();
                        File child = new File(parent, name);
                        if (!child.exists()) {
                            child.mkdirs();
                            if (child.isDirectory()) {
                                TreeItem item = new TreeItem(sel, SWT.NONE);
                                item.setText(child.getName());
                                item.setData(child);
                                item.setImage(PlatformUI.getWorkbench()
                                        .getSharedImages()
                                        .getImage(ISharedImages.IMG_OBJ_FOLDER));
                                tree.setFocus();
                            }
                        }
                    }
                }
            }
        };

        makeBut = new Button(mkDir, SWT.NONE);
        makeBut.setText("Make");
        makeBut.addSelectionListener(makeClick);

        this.getShell().setText(TITLE);
        this.getShell().setImage(
                PlatformUI.getWorkbench().getSharedImages()
                        .getImage(ISharedImages.IMG_OBJ_ELEMENT));
        return container;
    }

    protected Composite createClientContainer(Composite parent, int span,
            int step) {
        Composite container = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        if (step < 0)
            step = 10;
        layout.marginWidth = layout.marginHeight = step;
        layout.verticalSpacing = 2;
        layout.numColumns = span;
        container.setLayout(layout);
        return container;
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
                false);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }
}
