/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.editors.browser;

import java.util.UUID;

import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;

import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.nodetypes.Location;

public class ReqDocBrowser extends Composite {
    private Browser webBrowser;

    private static final boolean DEBUG = true;
    private boolean opend = false;

    public boolean isInitialized() {
        return opend;
    }

    public ReqDocBrowser(Composite parent, int style) {
        super(parent, style);

        this.setLayout(new FillLayout());
        webBrowser = new Browser(this, SWT.MOZILLA);

        webBrowser.addProgressListener(new ProgressListener() {
            @Override
            public void completed(ProgressEvent event) {
                if (!opend) {
                    injectScripts();
                    opend = true;
                }
            }

            @Override
            public void changed(ProgressEvent event) {
            }
        });
        webBrowser.addLocationListener(new LocationListener() {
            
            @Override
            public void changing(LocationEvent event) {
                if(opend)
                    event.doit=false;
            }
            
            @Override
            public void changed(LocationEvent event) {
            }
        });
    }

    protected boolean executeJS(String script) {
        if (DEBUG) {
            return webBrowser.execute("try{" + script
                    + "}catch(e){alert(e);alert(\"" + script + "\");}");
        } else {
            return webBrowser.execute(script);
        }
    }

    protected Object evaluateJS(String script) {
        if (DEBUG) {
            return webBrowser.evaluate("try{" + script
                    + "}catch(e){alert(e);alert(\"" + script + "\");}");
        } else {
            return webBrowser.evaluate(script);
        }
    }

    public String serializeDocument() {
        return (String) evaluateJS("return serializeDocument();");
    }

    public String serializeDocumentRaw() {
        return (String) evaluateJS("return serializeDocumentRaw();");
    }

    protected void injectScripts() {
        // TODO check if <head> element is found
        String jsPath = Activator.getDefault().getJSToolsPath();
        String script = "var h = document.getElementsByTagName('head')[0];"
                + "var s1 = document.createElement('script');"
                + "s1.setAttribute('id','jquery_js');"
                + "s1.setAttribute('type','text/javascript');"
                + "s1.setAttribute('src','" + jsPath + "jquery.js');"
                + "var s2 = document.createElement('script');"
                + "s2.setAttribute('id','edit_js');"
                + "s2.setAttribute('type','text/javascript');"
                + "s2.setAttribute('src','" + jsPath + "edit.js');"
                + "var s3 = document.createElement('link');"
                + "s3.setAttribute('id','edit_css');"
                + "s3.setAttribute('rel','stylesheet');"
                + "s3.setAttribute('type','text/css');"
                + "s3.setAttribute('href','" + jsPath + "edit.css');"
                + "h.appendChild(s1);" + "h.appendChild(s2);"
                + "h.appendChild(s3);" + "";
        executeJS(script);
    }

    public void selectNode() {
        executeJS("selectNode();");
    }

    public boolean areIdentifiersVisible() {
        return (Boolean) evaluateJS("return areIdentifiersVisible();");
    }

    public void toggleIdentifiersVisibility() {
        if (areIdentifiersVisible()) {
            executeJS("hideIdentifiers();");
        } else {
            executeJS("showIdentifiers();");
        }
    }

    public void showBrowserInfo() {
        executeJS("alert(getBrowserInfo());");
    }

    public boolean isSelectable() {
        return (Boolean) evaluateJS("return isSelectable();");
    }

    public boolean isRemovable() {
        return (Boolean) evaluateJS("return isRemovable();");
    }

    public String createLocation(String id, String reqId) {
        return (String) evaluateJS("return createLocation('" + id + "','"
                + reqId + "');");
    }

    public void deleteLocation(UUID locationUUID) {
        executeJS("removeLocation('" + locationUUID + "');");
    }

    public String getSelectedLocationUUID() {
        return (String) evaluateJS("return getSelectedLocationUUID();");
    }

    private String url;

    public void setUrl(String url) {
        webBrowser.setUrl(url);
        this.url = url;
    }

    public Browser getWebBrowser() {
        return webBrowser;
    }

    @Override
    public boolean setFocus() {
        return webBrowser.setFocus();
    }

    /**
     * @param targetRequirement
     *            - Target requirement or null
     */
    public void setTargetLocation(Location location) {
        if (location != null && opend) {
            webBrowser.setUrl(url + "#" + location.getId());
        }
    }

    public void updateLocations() {
        executeJS("removeAllLinks();");
        executeJS("addLocationLinks();");
    }
}
