package com.unitesk.requality.eclipse.views;


import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.ui.navigator.CommonDropAdapter;
import org.eclipse.ui.navigator.CommonDropAdapterAssistant;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class CNFdropAssistant extends CommonDropAdapterAssistant {
    private TreeNode srcNode = null;

    public CNFdropAssistant() {
        // TODO Auto-generated constructor stub
    }

    @Override
    public IStatus validateDrop(Object target, int operation,
            TransferData transferType) {
        if (!isSupportedType(transferType)) {
            return Status.CANCEL_STATUS;
        }

        if (target instanceof TreeNode) {
            LocalSelectionTransfer transfer = LocalSelectionTransfer
                    .getTransfer();
            ITreeSelection selection = (ITreeSelection) transfer.getSelection();
            if (selection.getFirstElement() instanceof TreeNode) {
                srcNode = (TreeNode) selection.getFirstElement();
            } else {
                return Status.CANCEL_STATUS;
            }

            TreeNode dstNode = (TreeNode) target;

            if (!srcNode.getTreeDB().equals(dstNode.getTreeDB())) {
                return Status.CANCEL_STATUS;
            }

            if (srcNode.getType().equals("Location")) {
                return Status.CANCEL_STATUS;
            }

            if (srcNode.getType().equals("Requirement")) {
                if (!dstNode.getType().equals("Requirement")) {
                    return Status.CANCEL_STATUS;
                }
            }

            if (srcNode.getType().equals("Document")) {
                if (!dstNode.getType().equals("DocFolder")) {
                    return Status.CANCEL_STATUS;
                }
            }

            if (srcNode.getType().equals("DocFolder")) {
                if (!dstNode.getType().equals("DocFolder")) {
                    return Status.CANCEL_STATUS;
                }
            }

            // we can't deep node
            for (TreeNode node : new DeepFirstTreeWalker(srcNode)) {
                if (node.getQualifiedId().equals(dstNode.getQualifiedId()))
                    return Status.CANCEL_STATUS;
            }
            // TODO: name check
            return Status.OK_STATUS;
        }
        return Status.CANCEL_STATUS;
    }

    @Override
    public IStatus handleDrop(CommonDropAdapter aDropAdapter,
            DropTargetEvent aDropTargetEvent, Object aTarget) {
        if (srcNode == null) {
            return Status.CANCEL_STATUS;
        }
        if (!(aTarget instanceof TreeNode)) {
            return Status.CANCEL_STATUS;
        }
        TreeNode target = (TreeNode) aTarget;
        if(target.findChild(srcNode.getId())!=null) {
            MessageDialog.openError(getShell(), "Error", "Child with id "+srcNode.getId()+" exists in "+target.getQualifiedId());
            return Status.CANCEL_STATUS;
        }
        srcNode.setParent(target);
        return Status.OK_STATUS;
    }
}
