/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.actions;

import java.io.File;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.NodeTypeDescriptor;
import com.unitesk.requality.eclipse.dialogs.ReportGenDialog;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.report.ReportGenerator;

public class GenerateReportAction extends Action {
    protected final static String GEN_REPORT = "Generate Report...";
    ISelectionProvider provider;
//    protected TreesView view;

    public GenerateReportAction() {
        throw new UnsupportedOperationException("use GenerateReportAction(TreesView tree)");
    }

    public GenerateReportAction(ISelectionProvider provider) {
        super(GEN_REPORT);
        setToolTipText(GEN_REPORT);
        setImageDescriptor(PlatformUI.getWorkbench()
                .getSharedImages()
                .getImageDescriptor(ISharedImages.IMG_OBJ_FILE));
//        this.view = view;
        this.provider = provider;
    }

    public void run() {
        ITreeSelection selected = (ITreeSelection) provider.getSelection();
        Object obj = selected.getFirstElement();
        TreeDB db = null;
        Requirement req=null;
        if (obj instanceof TreeDB)
            db = (TreeDB) obj;
        if (obj instanceof Requirement)
            req = ((Requirement) obj);
            db = req.getTreeDB();
        if (obj instanceof IFile)
            db = TreesTracker.getTracker().getTree(((IFile)obj).getProject());
        if (db != null) {
            ReportGenDialog dialog = new ReportGenDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), db);
            if(req!=null) {
                dialog.setStartNode(req);
            }
            if (dialog.open() == ReportGenDialog.OK) {
                TreeDB instTree = new TreeDB(new TreeInstancer(db));
                Map<String, NodeTypeDescriptor> nts = Activator.getAdditionalTypes();
                for(String name: Activator.getAdditionalTypes().keySet()) {
                    instTree.addNodeType(name, nts.get(name).getClazz());
                }

                String href = null;
                try {
                    href = ReportGenerator.newInstance().generateReport(
                            instTree, dialog.getTemplate(), dialog.getData(),
                            dialog.getSaveDir());
                } catch (Exception e) {
                    e.printStackTrace();
                }

                IWorkbenchPage page = PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getActivePage();
                if ((page != null) && (href != null) && (href.length() > 0)) {
                    try {
                        PlatformUI.getWorkbench().getBrowserSupport()
                                .createBrowser("req.report")
                                .openURL(new File(href).toURI().toURL());
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    @Override
    public boolean isEnabled() {
        ITreeSelection selection = (ITreeSelection) provider.getSelection();
        if(selection.isEmpty()) {
            return false;
        }
        Object obj = selection.getFirstElement();
        if (obj instanceof TreeNode) {
            TreeNode node = (TreeNode) obj;
            if(node.getType().equals("Requirement")) {
                return true;
            }
            
        }
        return false;
    }

}
