/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.actions.documents;

import java.io.File;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.documents.DocumentImporter;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.dialogs.ImportDialog;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.eclipse.views.actions.CNFAction;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Requirement;

public class ImportAction extends CNFAction {
    private static final String IMPORT = "Import Document...";
    private static final String MES_REPLACE = "File with same name already exists. Do you want to replace it?";

    public static String initalPath; // stores last open path

    public ImportAction(ISelectionProvider provider) {
//        super(provider);
        setText(IMPORT);
        setToolTipText(IMPORT);
        String IMG_ETOOL_IMPORT_WIZ = "IMG_ETOOL_IMPORT_WIZ"; // This is hack.
        setImageDescriptor(PlatformUI.getWorkbench().getSharedImages()
                .getImageDescriptor(IMG_ETOOL_IMPORT_WIZ));
    }

    public void run() {
        try {
            runAction();
        } catch (CoreException e) {
            e.printStackTrace();
        }
    }

    private void runAction() throws CoreException {
        DocFolder docFolder = getDocFolder();
        if (docFolder == null)
            return;

        ImportDialog dialog = new ImportDialog(docFolder);
        if (dialog.open() != ImportDialog.OK)
            return;

        String path = dialog.getPath();
        File inFile = new File(path);
        initalPath = inFile.getParent();

        // if (inFile.getAbsolutePath().startsWith(
        // docFolder.getTreeDB().getDocPath())) {
        // MessageDialog.openInformation(view.getSite().getShell(), "Import",
        // MES_ERRPLACE);
        // return;
        // }

        String newFileName = inFile.getName();
        newFileName = newFileName.substring(0, newFileName.lastIndexOf("."))
                + ".xhtml";
        Document doc = (Document) docFolder
                .createChild(newFileName, "Document");
        doc.setResourceName(newFileName);
        doc.saveAttributes();
        ResourceStorage rs = (ResourceStorage) doc.getTreeDB().getStorage();
        IFolder resFolder = rs.getResourceFolder(doc.getUUId());
        resFolder.create(false, true, null);

        File outFile = new File(resFolder.getLocation().toOSString(),
                newFileName);
        if (!outFile.exists()
                || outFile.equals(inFile)
                || MessageDialog.openConfirm(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Import",
                        MES_REPLACE)) {

            DocumentImporter din = new DocumentImporter();
            try {
                TreeDB db = docFolder.getTreeDB();
                String docId = doc.getQualifiedId();
                din.importDocument(docId, inFile, outFile, db,
                        (Requirement) db.getNode(Requirement.getTypeRootQId()),
                        dialog.getEncoding());
                IProject proj = TreesTracker.getTracker().getProject(db);
                proj.refreshLocal(IResource.DEPTH_INFINITE, null);
                openDocument(docId, db);
                Activator.getDefault().setActiveNode(doc, doc.getType());
            } catch (Exception e) {
                String message = e.getMessage().length() > 0? e.getMessage():"Document processing error";
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(),
                        "Document processing error", message);
            }
        }
    }

    public static void openDocument(String docId, TreeDB db) {
        Document doc = (Document) db.getNode(docId);
        IWorkbenchPage page = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage();
        if (page != null) {
            try {
                page.openEditor(new NodeEditorInput(doc),
                        "com.unitesk.requality.editors.browser.ReqMarker", true);
            } catch (PartInitException e) {
                e.printStackTrace();
            }
        }
    }

    private DocFolder getDocFolder() {
        TreeNode obj = getSelectedNode();

        if (obj instanceof DocFolder) {
            return (DocFolder) obj;
        }
        return null;
    }

    @Override
    public boolean isEnabled() {
        return getDocFolder() != null;
    }
}
