/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.views.requirements;

import java.io.File;

import org.eclipse.core.resources.IProject;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.nodetypes.Requirement;

public class ReqViewLabelProvider extends LabelProvider implements
        ILabelProvider {
    public Image getImage(Object obj) {
        String imageId = ISharedImages.IMG_OBJ_ELEMENT;
        if (obj instanceof TreeDB)
            imageId = IDE.SharedImages.IMG_OBJ_PROJECT;

        if (obj instanceof TreeNode) {
            TreeNode node = (TreeNode) obj;
            if (node.getQualifiedId().equals(
                    node.getTreeDB().getNode(Requirement.getTypeRootQId())
                            .getQualifiedId()))
                return getImage(node.getTreeDB());

            return Activator.getDefault().getImageRegistry().get("nodetype."+node.getType());
        }
        if (obj instanceof String)
            imageId = ISharedImages.IMG_OBJS_INFO_TSK;
        if (obj instanceof File)
            imageId = ISharedImages.IMG_OBJ_FILE;
        return PlatformUI.getWorkbench().getSharedImages().getImage(imageId);
    }

    public String getText(Object element) {
        if (element != null) {
            if (element instanceof TreeDB) {
                TreeDB db = (TreeDB) element;
                IProject proj = TreesTracker.getTracker().getProject(db);
                if (proj == null)
                    return element.toString();
                return TreesTracker.getTracker().getProject(db).getName();
            } 
            if (element instanceof Requirement) {
                Requirement node = (Requirement) element;

                if (node.equals(node.getTreeDB().getNode(
                        Requirement.getTypeRootQId())))
                    return getText(node.getTreeDB());
                
                return node.getName();
            }
            if (element instanceof TreeNode) {
                TreeNode node = (TreeNode) element;

                if (node.equals(node.getTreeDB().getNode(
                        Requirement.getTypeRootQId())))
                    return getText(node.getTreeDB());
                
                return node.getId();
            }
        }
        return String.valueOf(element);
    }
}
