/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package com.unitesk.requality.eclipse.wizards;

import java.io.File;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.documents.DocumentImporter;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.tools.ResourceStorage;
import com.unitesk.requality.eclipse.tools.TreesTracker;
import com.unitesk.requality.eclipse.views.actions.documents.ImportAction;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Requirement;

public class DocumentImportWizard extends Wizard implements IImportWizard {
//    private static final String MES_ERRPLACE = "File to import sould not be situated in the system folder.";
    private static final String MES_REPLACE = "File with same name already exists. Do you want to replace it?";

    DocumentImportWizardPage mainPage;

    public DocumentImportWizard() {
        super();
    }

    public boolean performFinish() {
        String path = mainPage.getImportFileName();
        File inFile = new File(path);
        try {
            String newFileName = inFile.getName();
            newFileName = newFileName
                    .substring(0, newFileName.lastIndexOf(".")) + ".xhtml";
            Document doc = (Document) mainPage.getDocumentParent().createChild(
                    newFileName, "Document");
            doc.setResourceName(newFileName);
            doc.saveAttributes();
            ResourceStorage rs = (ResourceStorage) doc.getTreeDB().getStorage();
            IFolder resFolder = rs.getResourceFolder(doc.getUUId());
            resFolder.create(false, true, null);

            File outFile = new File(resFolder.getLocation().toOSString(),
                    newFileName);

            if (!outFile.exists()
                    || outFile.equals(inFile)
                    || MessageDialog.openConfirm(PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getShell(), "Import",
                            MES_REPLACE)) {

                DocumentImporter din = new DocumentImporter();
                TreeDB db = mainPage.getSelectedDB();
                String docId = doc.getQualifiedId();
                din.importDocument(docId, inFile, outFile, db,
                        (Requirement) db.getNode(Requirement.getTypeRootQId()),
                        "UTF-8");
                Activator.getDefault().setActiveNode(doc, doc.getType());
                // TODO: choose encoding from GUI
                IProject proj = TreesTracker.getTracker().getProject(db);
                proj.refreshLocal(IResource.DEPTH_INFINITE, null);
                ImportAction.openDocument(docId, db);
            }
        } catch (Exception e) {
            MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(),
                    "Document processing error", e.getMessage());
        }

        //
        // DocFolder docFolder = new DocFolder(mainPage.getSelectedDB(), new
        // File(
        // mainPage.getSelectedFolder()));
        //

        // File inFile = new File(path);
        // if (isFileInBase(inFile)) {
        // MessageDialog.openInformation(getShell(), "Import", MES_ERRPLACE);
        // return false;
        // }
        //
        // String newFileName = inFile.getName();
        // newFileName = newFileName.substring(0, newFileName.lastIndexOf("."))
        // + ".xhtml";
        //
        // File outFile = new File(mainPage.getSelectedFolder(), newFileName);
        // if (!outFile.exists() || outFile.equals(inFile)
        // || MessageDialog.openConfirm(getShell(), "Import", MES_REPLACE)) {
        //
        // DocumentImporter din = new DocumentImporter();
        // try {
        // TreeDB db = docFolder.getTreeDB();
        // String docId = docFolder.getRelativePath() + File.separator
        // + outFile.getName();
        // din.importDocument(docId, inFile, outFile, db,
        // (Requirement) db.getNode(Requirement.getTypeRootQId()),
        // "UTF-8");
        // // TODO: choose encoding from GUI
        // IProject proj = TreesTracker.getTracker().getProject(
        // docFolder.getTreeDB());
        // proj.refreshLocal(IResource.DEPTH_INFINITE, null);
        // } catch (IOException e) {
        // MessageDialog.openError(getShell(),
        // "Document processing error", e.getMessage());
        // } catch (CoreException e) {
        // e.printStackTrace();
        // }
        // }
        return true;
    }

    public void init(IWorkbench workbench, IStructuredSelection selection) {
        setWindowTitle("Document Import Wizard");
        setNeedsProgressMonitor(false);
        mainPage = new DocumentImportWizardPage();
    }

    public void addPages() {
        super.addPages();
        addPage(mainPage);
    }

    @Override
    public boolean canFinish() {
        return mainPage.isDataReady() && super.canFinish();
    }
}
