package com.unitesk.requality.merge;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.w3c.dom.Document;

public class Tree {

    private List<Node> levelOrderedTraversal;

    protected Tree() {
	levelOrderedTraversal = new ArrayList<Node>();
    }

    public Tree(org.w3c.dom.Node domNode) {
	makeLevelOrderedTraversalSequence(domNode);
    }

    public Tree(Document doc) {
	makeLevelOrderedTraversalSequence(doc.getDocumentElement());
    }

    protected void makeLevelOrderedTraversalSequence(org.w3c.dom.Node root) {
	levelOrderedTraversal = new ArrayList<Node>();
	levelOrderedTraversal.add(new Node(root));
	for (int i = 0; i < levelOrderedTraversal.size(); i++) {
	    Node currentNode = levelOrderedTraversal.get(i);
	    if (!currentNode.getDomNode().hasChildNodes())
		continue;
	    for (org.w3c.dom.Node x = currentNode.getDomNode().getFirstChild(); x
		    .getNextSibling() != null; x = x.getNextSibling())
		levelOrderedTraversal.add(new Node(currentNode, x));
	    levelOrderedTraversal.add(new Node(currentNode, currentNode
		    .getDomNode().getLastChild()));
	}
    }

    public Iterator<Node> traverse() {
	return levelOrderedTraversal.listIterator();
    }

    public List<Node> getTraversalSequence() {
	return levelOrderedTraversal;
    }

    public Node getRoot() {
	return levelOrderedTraversal.get(0);
    }

    public Node nearestMatch(Node x) {
	return nearestMatch(x, this.levelOrderedTraversal);
    }
    
    public Node nearestMatch(Node x, NodeFilter f) {
	return nearestMatch(x, f.filter(this.levelOrderedTraversal));
    }
    
    public Node nearestMatch(Node x, Node parent) {
	return nearestMatch(x, parent.getChilds());
    }
    
    public Node nearestMatch(Node x, Node parent, NodeFilter f) {
	return nearestMatch(x, f.filter(parent.getChilds()));
    }
    
    private Node nearestMatch(Node x, List<Node> possibleMatches) {
	int min_distance = Integer.MAX_VALUE;
	Node nearestMatch = null;
	Context refContext = x.getContext();
	for (Node y : possibleMatches) {
	    Context currContext = y.getContext();
	    int distance;
	    if(!x.isLeaf() && !y.isLeaf())
		distance = refContext.contextualDistance(currContext);
	    else
		distance = refContext.positionalDistance(currContext);
	    if (distance < min_distance) {
		min_distance = distance;
		nearestMatch = y;
	    }
	}
	return nearestMatch;
    }

    public boolean isEqual(Tree t) {
	if (this.levelOrderedTraversal.size() != t.levelOrderedTraversal.size())
	    return false;
	Iterator<Node> i = this.traverse();
	Iterator<Node> j = t.traverse();
	while (i.hasNext() && j.hasNext()){
	    Node reference = i.next();
	    Node match = j.next();
	    if (!reference.isEqual(match))
		return false;
	}
	return true;
    }

    public void updateTree() {
	makeLevelOrderedTraversalSequence(getRoot().getDomNode());
    }
}
