package com.unitesk.requality.tests.core;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.MemoryStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.instancer.TreeInstancer;

public class InstanceTreeTest {
    private TreeDB treedb;
    private TreeNode req0;

    @Before
    public void init() {
        treedb = new TreeDB(new MemoryStorage());

        TreeNode root = treedb.getRootNode();
        req0 = root.createChild("R0");
        req0.putAttribute(new Attribute(AttributeType.STRING, "d", "1"));
        req0.setPredicate("d < 3");
        req0.saveAttributes();
        TreeNode req1 = req0.createChild("R1");
        TreeNode req2 = req0.createChild("R2");
        req2.setPredicate("d < 2");
        req2.saveAttributes();

        TreeNode req21 = req2.createChild("R21");
        req21.putAttribute(new Attribute(AttributeType.STRING, "d", "2"));
        req21.saveAttributes();
        req2.createChild("R22").saveAttributes();
        req1.createChild("R11").saveAttributes();
        req1.addBaseNode(req2.getQualifiedId());
        req1.saveAttributes();
    }

    @After
    public void tearDown() {
    }

    @Test
    public void createTreeWithBaseReq() {
        TreeDB newTree = new TreeDB(new TreeInstancer(treedb));
        assertNotNull(newTree.getRootNode());
        TreeNode instRoot = newTree.getRootNode().getChild(0);
        assertNotNull(instRoot);

        assertEquals("У узла должно быть 2 ребенка", 2, instRoot.sizeChildren());
        TreeNode instNode0 = instRoot.getChild(0);
        TreeNode instNode1 = instRoot.getChild(1);
        assertTrue("Ребенок R0 имеет VisibleId - R1 или R2",
                (instNode0.getId().equals("R1")) || (instNode0.getId().equals("R2")));
        assertTrue("Ребенок R0 имеет VisibleId - R1 или R2",
                (instNode1.getId().equals("R1")) || (instNode1.getId().equals("R2")));
        assertFalse("VisibleId у детей R0 разные",
                instNode0.getId().equals(instNode1.getId()));
        // определяем, какой узел R1, а какой R2
        TreeNode instNodeR1;
        TreeNode instNodeR2;
        if (instNode0.getId().equals("R1")) {
            instNodeR1 = instNode0;
            instNodeR2 = instNode1;
        } else {
            instNodeR1 = instNode1;
            instNodeR2 = instNode0;
        }

        assertEquals("У R1 должно быть два ребенка", 2,
                instNodeR1.sizeChildren());
        instNode0 = instNodeR1.getChild(0);
        instNode1 = instNodeR1.getChild(1);
        assertTrue("Ребенок R1 имеет VisibleId - R11 или R22",
                (instNode0.getId().equals("R11")) || (instNode0.getId().equals("R22")));
        assertTrue("Ребенок R1 имеет VisibleId - R11 или R22",
                (instNode1.getId().equals("R11")) || (instNode1.getId().equals("R22")));
        assertFalse("VisibleId у детей R1 разные",
                instNode0.getId().equals(instNode1.getId()));
        // определяем, какой узел R11, а какой R22
        TreeNode instNodeR11;
        TreeNode instNodeR22;
        if (instNode0.getId().equals("R11")) {
            instNodeR11 = instNode0;
            instNodeR22 = instNode1;
        } else {
            instNodeR11 = instNode1;
            instNodeR22 = instNode0;
        }
        assertEquals("У R2 должен быть один ребенок", 1,
                instNodeR2.sizeChildren());
        TreeNode instNodeR22_2 = instNodeR2.getChild(0);
        assertEquals("Ребенок R2 VisibleId - R22", "R22", instNodeR22_2.getId());

        assertEquals("У R11 не должно быть детей", 0,
                instNodeR11.sizeChildren());
        assertEquals("У R22 не должно быть детей", 0,
                instNodeR22.sizeChildren());
        assertEquals("У R22 (ребенок R2) не должно быть детей", 0,
                instNodeR22_2.sizeChildren());
    }

    @Test
    public void predicateInit() {
        TreeDB db = new TreeDB(new MemoryStorage());
        TreeNode root = db.getRootNode();
        root.setPredicate("true==true");
        TreeNode a = root.createChild("A");
        a.setPredicate("true");
        TreeDB insTree = new TreeDB(new TreeInstancer(db));
        TreeInstancer ti = (TreeInstancer) insTree.getStorage();
        String newPredicate = ti.getPredicate(a.getUUId());
        assertEquals("(true==true) && (true)", newPredicate);
    }
}
